#include "level_menu.h"

#include "map.h"
#include "game_display.h"
#include "info.h"
#include "scores.h"
#include "screen.h"

#include <curses.h>
#include <string.h>
#include <stdlib.h>

#define LVL_NAME_CHARS 24

void level_menu_draw();

void splash();

void icon_help();

#ifdef TESTMAP
su_t min_level = 0;
#else
su_t min_level = 1;
#endif

su_t max_level = 15;
su_t sel_level = 1;

lvl_t
level_menu(lvl_t level)
{
    wclear(info_win);
    screen_data->game_win_repaint_cb = &splash;
    screen_data->info_win_repaint_cb = &level_menu_draw;
    sel_level = level;
    splash();
    do {
        level_menu_draw();
        int key = wgetch(info_win);
        if (key != ERR) {
            switch (key) {
            case KEY_RESIZE:
                screen_resize();
                break;
            case '\'':
            case KEY_UP:
                if (sel_level > min_level)
                    sel_level--;
                break;
            case '/':
            case KEY_DOWN:
                if (sel_level < max_level)
                    sel_level++;
                break;
            case 'd':
                options->oldschool_play ^= 1;
                if (options->oldschool_play)
                    options->scroll_thresh = 1;
                else
                    options->scroll_thresh = 2;
                screen_resize();
                break;
            case 'q':
            case 'Q':
                return -1;
            case '\r':
                return sel_level;
            case 'h':
            case 'H':
                return 999;
            case 'l':
            case 'L':
                return 100;
            default:
                if (key >= '0' && key <= '9') {
                    su_t i = 9 - ('9' - key);

                    if (i >= min_level && i <= max_level)
                        sel_level = i;
                }
                break;
            }
        }
    } while (1);
}

void
level_menu_draw()
{
    wattrset(info_win, COLOR_PAIR(0));
    box(info_win, 0, 0);

    mvwprintw(info_win, 1, 3, "Select level:");
    for (su_t i = min_level; i < MAX_LEVEL; i++) {
        if (i == sel_level)
            wattrset(info_win, COLOR_PAIR(ICON_XXX + 1));
        else if (i <= max_level)
            wattrset(info_win, COLOR_PAIR(0));
        else
            wattrset(info_win, COLOR_PAIR(ICON_MASK));
        mvwprintw(info_win, 2 + i, 1, " %2d. %s ", i, map_name[i]);
        wattrset(info_win, COLOR_PAIR(0));
        mvwprintw(info_win, 2 + i, MAPNAME_MAXCHARS + 2, "%4d", scores[i]);
    }
    mvwprintw(info_win, 19, 4, "q - quit");
    mvwprintw(info_win, 20, 4, "l - load replay");
    mvwprintw(info_win, 21, 4, "d - difficulty: %s",
              (options->oldschool_play ? "old school" : "new school"));
    mvwprintw(info_win, 22, 4, "h - help");
    wrefresh(info_win);
}

void
splash()
{
    NCURSES_COLOR_T mc_bg = COLOR_BLUE;

    NCURSES_COLOR_T mc_fg = COLOR_BLACK;

    char *xortitle[] = {
        "--  --  ---  ---\\",
        " \\  /  /   \\ |   |",
        "  \\/   |   | |--/",
        "  /\\   |   | |  \\",
        " /  \\  \\   / |   \\",
        "--  --  ---  - Curses",
        "X"
    };
    char *xormask[] = {
        " ___*******___",
        "/   -------   \\",
        "|             |",
        "\\  ---\\ /---  /",
        " | \\--- ---/ |",
        "/             \\",
        "|             |",
        "\\  \\\\     //  /",
        " \\  \\\\   //  /",
        "  \\  \\\\ //  /",
        "   \\  \\_/  /",
        "    \\     /",
        "     \\___/",
        "X"
    };
    char *xorlns[] = {
        "           ---           ---",
        "         ------         ------",
        "      ----------       ----------",
        "",
        " -------------------------------------",
        "  XorCurses-0.0.9      by jwm-art.net",
        "X"
    };
    char **ptr1;

    char *ptr2;

    xy_t x;

    xy_t y;

    xy_t offx = ((screen_data->garea_w - 8) / 2) * ICON_W;
    xy_t offy = ((screen_data->garea_h - 8) / 2) * ICON_H + 1;

    bool msk = FALSE;

    wclear(game_win);
    init_pair(ICON_WALL, COLOR_WHITE, COLOR_RED);       /* for Xor title */
    init_pair(ICON_XXX, mc_fg, mc_bg);
    ptr1 = xortitle;
    ptr2 = ptr1[0];
    y = offy + 0;
    while (*ptr1[0] != 'X') {
        ptr2 = *ptr1;
        x = offx + 10;
        while (*ptr2) {
            if (*ptr2 != ' ')
                wattrset(game_win, COLOR_PAIR(ICON_WALL));
            else
                wattrset(game_win, COLOR_PAIR(0));
            mvwaddch(game_win, y, x, *ptr2);
            x++;
            ptr2++;
        }
        ptr1++;
        y++;
    }
    ptr1 = xormask;
    ptr2 = ptr1[0];
    y = offy + 7;
    wattrset(game_win, COLOR_PAIR(0));
    while (*ptr1[0] != 'X') {
        ptr2 = *ptr1;
        x = offx + 12;
        msk = FALSE;
        while (*ptr2) {
            if (*ptr2 != ' ') {
                wattrset(game_win, COLOR_PAIR(ICON_SWITCH));
                msk = TRUE;
            }
            else if (msk == TRUE)
                wattrset(game_win, COLOR_PAIR(ICON_XXX));
            if (*ptr2 != '*')
                mvwaddch(game_win, y, x, *ptr2);
            x++;
            ptr2++;
        }
        ptr1++;
        y++;
    }
    ptr1 = xorlns;
    ptr2 = ptr1[0];
    y = offy + 16;
    wattrset(game_win, COLOR_PAIR(0));
    while (*ptr1[0] != 'X') {
        ptr2 = *ptr1;
        x = offx + 0;
        while (*ptr2) {
            if (*ptr2 != ' ')
                mvwaddch(game_win, y, x, *ptr2);
            x++;
            ptr2++;
        }
        ptr1++;
        y++;
    }
    wrefresh(game_win);
}

void
help()
{
    char *hf = options_file_path("xorcurseshelp.txt", options->data_dir);

    if (!hf)
        return;
    FILE *fp = fopen(hf, "r");

    free(hf);
    if (!fp) {
        icon_help();
        return;
    }
    xy_t row = 0;

    char help[120][43];

    while (fgets(help[row++], 42, fp) && row < 120);
    fclose(fp);
    xy_t maxrows = row - 1;

    xy_t h = screen_data->garea_h * ICON_H;

    row = 0;
    wclear(game_win);
    wattrset(game_win, COLOR_PAIR(0));
    int key;

    void (*cur_repaint_cb) () = screen_data->game_win_repaint_cb;

    screen_data->game_win_repaint_cb = 0;

    do {
        for (xy_t r = 0; r < h; r++)
            mvwprintw(game_win, r, 0, help[r + row]);
        switch ((key = wgetch(game_win))) {
        case KEY_RESIZE:
            screen_resize();
            xy_t oh = h;
            h = screen_data->garea_h * ICON_H;
            if (h > oh)
                if (row + h > maxrows)
                    row -= (row + h) - maxrows;
            break;
        case KEY_UP:
            if (row > 0)
                row--;
            break;
        case KEY_DOWN:
            if (row + h < maxrows)
                row++;
            break;
        case 'q':
        case 'Q':
            screen_data->game_win_repaint_cb = cur_repaint_cb;
            return;
        }
    } while (key != 'h' && key != 'H');
    icon_help();
    screen_data->game_win_repaint_cb = cur_repaint_cb;
}

void
icon_help()
{
    char *hf = options_file_path("xorcursesiconhelp.txt", options->data_dir);

    if (!hf)
        return;
    FILE *fp = fopen(hf, "r");

    free(hf);
    if (!fp)
        return;
    xy_t row = 0;

    xy_t h = screen_data->garea_h * ICON_H;

    char help[50][43];

    while (fgets(help[row++], 42, fp) && row < 50);
    fclose(fp);
    xy_t maxrows = row - 1;

    wclear(game_win);
    row = 0;
    int key;

    do {
        wattrset(game_win, COLOR_PAIR(0));
        for (xy_t r = 0; r < h; r++)
            mvwprintw(game_win, r, 0, help[r + row]);
        for (su_t i = 0; i < 12; i++)
            game_win_icon_dump(0, i * 4 - row, i + ICON_H_FIELD);
        switch ((key = wgetch(game_win))) {
        case KEY_RESIZE:
            screen_resize();
            xy_t oh = h;
            h = screen_data->garea_h * ICON_H;
            if (h > oh)
                if (row + h > maxrows)
                    row -= (row + h) - maxrows;
            break;
        case KEY_UP:
            if (row > 0)
                row--;
            break;
        case KEY_DOWN:
            if (row + h < maxrows)
                row++;
            break;
        case 'q':
        case 'Q':
        case 'h':
        case 'H':
            return;
        }
    } while (key != 'q' && key != 'Q');
}
