#include "play_xor.h"
#include <stdlib.h>
#include <time.h>

#include <curses.h>

#ifdef DEBUG
#include <stdio.h>
#endif

#include "options.h"
#include "scores.h"
#include "screen.h"

void
info_win_repaint()
{
    wclear(info_win);
    wattrset(info_win, COLOR_PAIR(0));
    box(info_win, 0, 0);
    info_win_update_player_icon();
    info_win_update_map(player.have_map);
    info_win_display();
}

bool
play_xor(lvl_t level, bool from_start)
{
    player.replay = 0;
    if (from_start) {
        replay.level = level;
        for (ctr_t mv = MAX_MOVES; mv > 0; mv--)
            replay.moves[mv] = 0;
        init_wall(level, TRUE);
        xor_map_create();
        xor_map_load(level);
        player_init();
    }
    else
        level = replay.level;
    game_win_init_views();
    game_win_display();
    info_win_repaint();
    screen_data->game_win_repaint_cb = &game_win_display;
    screen_data->info_win_repaint_cb = &info_win_repaint;
    info_win_display();
    su_t state = PLAY_CONTINUE;

    while (state == PLAY_CONTINUE || state == PLAY_PROCESS_MOVE) {
        int key = wgetch(game_win);

        if (key != ERR) {
            su_t move = MV_NONE;

            switch (key) {
            case KEY_RESIZE:
                screen_resize();
                break;
            case KEY_LEFT:
            case 'z':
                move = MV_LEFT;
                break;
            case KEY_RIGHT:
            case 'x':
                move = MV_RIGHT;
                break;
            case KEY_UP:
            case '\'':
                move = MV_UP;
                break;
            case KEY_DOWN:
            case '/':
                move = MV_DOWN;
                break;
            case '\r':
            case 'p':
                move = MV_PLAYER_SWAP;
                break;
            case 'q':
            case 'Q':
                move = MV_PLAYER_QUIT;
                break;
                if (screen_data->scale_map == TRUE)
                    game_win_map_display();
                break;
            case '1':
                options->scroll_thresh = 1;
                break;
            case '2':
                options->scroll_thresh = (options->oldschool_play ? 1 : 2);
                break;
            case '3':
                options->scroll_thresh = (options->oldschool_play ? 1 : 3);
                break;
            }
            if (move) {
                if ((state = player_move(move)) & PLAY_RECORD) {
                    replay.moves[player.moves_remaining--] = move;
                    state ^= PLAY_RECORD;
                }
                info_win_display();
                if ((!player.p0_alive) && (!player.p1_alive))
                    state = PLAY_GOTCHA;
            }
        }
        }
        if (state == PLAY_COMPLETE)
            save_score(map->level, MAX_MOVES - player.moves_remaining);
        if (player.moves_remaining < MAX_MOVES)
            return replay_xor(TRUE);
        return FALSE;
    }

bool replay_xor(bool ask)
{
    int key;

    su_t state;

    struct timespec rpause;

    struct timespec repause;

    bool continue_play = FALSE;

    bool ask_continue = player.p0_alive | player.p1_alive;

    screen_data->game_win_repaint_cb = &game_win_display;
    screen_data->info_win_repaint_cb = &info_win_repaint;

    rpause.tv_sec = 0;
    rpause.tv_nsec = options_replay_speed(options->replay_speed);
    do {
        if (ask == TRUE) {
            if (ask_continue)
                scr_wmsg(game_win, "Replay (Y/N/C/S)?", 0, 0);
            else
                scr_wmsg(game_win, "Replay (Y/N/S)?", 0, 0);
            key = wgetch(game_win);
            game_win_display();
            switch (key) {
            case KEY_RESIZE:
                screen_resize();
                break;
            case 'n':
            case 'N':
            case 'q':
            case 'Q':
                xor_map_destroy();
                return FALSE;
            case 's':
            case 'S':
                replay_save();
                xor_map_destroy();
                return FALSE;
            case 'c':
            case 'C':
                if (ask_continue)
                    if (player.p0_alive || player.p1_alive)
                        continue_play = TRUE;
            }
        }
        else {
            ask = TRUE;
            key = 'y';
        }
        if (key == 'y' || key == 'Y') {
            xor_map_create();
            xor_map_load(replay.level);
            player_init();
            player.replay = TRUE;
            init_wall(replay.level, TRUE);
            game_win_init_views();
            game_win_display();
            info_win_update_map(player.have_map);
            info_win_repaint();
            info_win_display();
            info_win_update_player_icon();
            nodelay(game_win, TRUE);
            state = PLAY_CONTINUE;
            while (state == PLAY_CONTINUE || state == PLAY_PROCESS_MOVE) {
                nanosleep(&rpause, &repause);
                if ((key = wgetch(game_win)) != ERR) {
                    if (key == KEY_RESIZE)
                        screen_resize();
                    else if (key == 'q' || key == 'Q')
                        state = player_move(MV_PLAYER_QUIT);
                    else if (key >= '1' && key <= '9')
                        rpause.tv_nsec = options_replay_speed(key);
                }
                else {
                    state = player_move(replay.moves[player.moves_remaining--])
                            ^ PLAY_RECORD;
                }
                info_win_display();
                    if ((!player.p0_alive) && (!player.p1_alive))
                        state = PLAY_GOTCHA;
            }
            nodelay(game_win, FALSE);
            if (state == PLAY_GOTCHA)
                ask_continue = FALSE;
            else
                ask_continue = TRUE;
        }
    } while (!continue_play);
    state = PLAY_CONTINUE;
    if (replay.moves[player.moves_remaining + 1] == MV_PLAYER_QUIT)
        player.moves_remaining++;
    return TRUE;
}

void replay_save()
{
    FILE *fp = 0;
    char *filename = scr_wmsg_read(game_win, "Filename:", 30);
    game_win_display();
    if ((fp = fopen(filename, "r"))) {
        scr_wmsg(game_win, "Exists! Overwrite (y/n)?", 0, 0);
        fclose(fp);
        int key = wgetch(game_win);
        game_win_display();
        if (key == 'n' || key == 'N') {
            free(filename);
            return;
        }
    }
    if (!(fp = fopen(filename, "w"))) {
        scr_wmsg(game_win, "Error!", 0, 0);
        goto bail;
    }
    if (!fwrite(&replay, sizeof(struct xor_replay), 1, fp)) {
        scr_wmsg(game_win, "Error!", 0, 0);
        goto bail;
    }
    scr_wmsg(game_win, "Saved!", 0, 0);

bail:
    wgetch(game_win);
    game_win_display();
    if (filename)
        free(filename);
    if (fp)
        fclose(fp);
}

lvl_t replay_load()
{
    FILE *fp = 0;
    char *filename = scr_wmsg_read(game_win, "Filename:", 30);
    screen_data->game_win_repaint_cb();
    if (!(fp = fopen(filename, "r"))) {
        scr_wmsg(game_win, "Open Error!", 0, 0);
        goto bail;
    }
    free(filename);
    if (!fread(&replay, sizeof(struct xor_replay), 1, fp)) {
        scr_wmsg(game_win, "Read Error!", 0, 0);
        goto bail;
    }
    fclose(fp);
    if (replay_xor(FALSE))
        return 100;
    return replay.level;

bail:
    wgetch(game_win);
    screen_data->game_win_repaint_cb();
    if (filename)
        free(filename);
    if (fp)
        fclose(fp);
    return -1;
}
